/**************************************************************************/
/*                                                                        */
/*   Program Name :  vssplogna                                            */
/*                                                                        */
/*      Version:  1.10  2016-05-16                                        */
/*                                                                        */
/*                                                                        */
/*      Log file analysis for PC-VSSP(IP-VLBI) board                      */
/*                                                                        */
/*                                                                        */
/*      Copyright (c) 2007-2016 T.Kondo/NICT All Right Reserved           */
/*                                                                        */
/**************************************************************************/
/*
      usage
             vssplogana log_file1 [log_file2 [....]]]
                  where   log_file1 -- datachk log file and/or sampling log file
                          log_file2 -- datachk log file and/or sampling log file

        ver 0.00 2007-02-08  development start
        ver 1.01 2007-02-14  term of "header separation error" is  changed to
		                     "AUX field in headder error"
        ver 1.02 2007-02-16  term of "BAD" is  changed to "SLIP"
        ver 1.03 2007-03-02  bit reversal error is added 
        Ver 1.04 2008-02-19  options '--help' and '--ver[sion]' were added
        Ver 1.10 2016-05-16  support VSSP64

D:/IPVLBI/utility/k5vssp32/check/unit_A60.log
D:/IPVLBI/utility/k5vssp32/check/unitAtest.log
*/
#define PROGRAM_NAME "vssplogana2"
#define PROGRAM_VERSION "Ver. 2016-09-01"

//#include "libk5.h"      // common header for IP-VLBI utilities

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __CYGWIN__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#ifdef WIN32
#pragma warning (disable:4996)
#endif

//#ifndef WIN32
//#include <stdint.h>
//#endif
//#define REWIND_BYTE  0  // define how many bytes rewind if failure in 
                          // header detection  on 1st trial (shoud be multiples of 4)
                          // 0 means rewind till previous header

//#define N123(a,b,c) (a+b*6+c*6*4)   //  macro to convert 3D array pointer to 1D array pointer
                                    //  for array(6,4,2)
#define N123(a,b,c) (a+b*7+c*7*4)   //  macro to convert 3D array pointer to 1D array pointer
                                    //  for array(7,4,3)
#define ARRAY_SIZE 84               //  7x4x3

#define MAX_SCAN 100     // maximum # of scans

typedef struct DATACHK_LOG_SCAN    // for datachk log analysis
{
	int k5format;  // 1: K5/VSSP    2: K5/VSSP32
    int adbit;  // AD bits
    int sfreq;  // sampling frequency (MHz)
    int nch;    // # of cahnnels
    int lpf;

	int frames;               // # of frames
	int slipfram;             // # of bad (bit slip or make) frames 
	int tdiscon;              // # of time discon frames
	int tdiscon2;             // # of time discon and bit slip or make frames
	int auxerr;               // # of AUX field error frames
	int auxmis;               // # of AUX field misalign occurred frames
	int bitrev;               // # of 1bit reversal in sync pattern error
	int eflg;                 // # of EFLG set frames
	int h1st;                 // # of frames where 1st 64bit is not a header
    int times[5];             // start time  yyyy, ddd, hh, mm, ss get from 1st header
    int slip;                 // # of slip frames
    int span;               // scan period (sec)
} DATACHK_LOG_SCAN_T; 

typedef struct DATACHK_LOG   // for datachk log analysis
{
    int nscan;                   // # of scan
    DATACHK_LOG_SCAN_T *scan;   // each scan data
} DATACHK_LOG_T;


typedef struct SAMPLING_LOG_SCAN              // for sampling log analysis
{
    int adbit;  // set AD bits
    int sfreq;  // set sampling frequency (MHz)
    int nch;    // set # of cahnnels
    int span;   // set span length (sec)
    int flag_lpf;  // 1: set LPF   0: not set

	int badscan; // # of scans where error occurred during sampling by sampling parameters
	int err16;   // # of error 16 occurrences during sampling by sampling parameters
	int err19;   // # of error 19 occurrences during sampling by sampling parameters
	int err20;   // # of error 20 occurrences during sampling by sampling parameters
	int others;  // # of other errors occurrences during sampling by sampling parameters
	int errall;  // # of total errors occurrences during sampling by sampling parameters
    int times[5];  // time (yyyy, ddd, hh, mm, ss) when "sampling" was executed
} SAMPLING_LOG_SCAN_T;

typedef struct SAMPLING_LOG   // for sampling log analysis
{
    int nscan;                   // # of scan
    SAMPLING_LOG_SCAN_T *scan;   // each scan data
} SAMPLING_LOG_T;

typedef struct VSSPLOGANA_INTERNAL_USE  // for vssplogana internal use
{
	char fname[120];
	FILE *fin;
	int k5format;  // 1: K5/VSSP    2: K5/VSSP32
	int logmode;  // logfile mode  1: sampling log  2: datachk log
	int spanmax;  // scan period max (sec)
	int spanmin;  // scan period min (sec)
	int nscantotal; // total number of scans
	int nscan[ARRAY_SIZE];  // # of scans by sampling parameter (sfreq,adbits,nch)
	int frames[ARRAY_SIZE]; // # of frames by sampling parameter (sfreq,adbits,nch)
	int slipfram[ARRAY_SIZE]; // # of bad (bit slip or make) frames by sampling parameter
	int tdiscon[ARRAY_SIZE]; // # of time discon frames by sampling parameter
	int tdiscon2[ARRAY_SIZE]; // # of time discon and bit slip or make frames by sampling parameter
	int auxerr[ARRAY_SIZE]; // # of AUX field error frames by sampling parameter
	int auxmis[ARRAY_SIZE]; // # of AUX field misalign occurred frames by sampling parameter
	int bitrev[ARRAY_SIZE]; // # of 1bit reversal in sync pattern error
	int eflg[ARRAY_SIZE]; // # of EFLG set frames by sampling parameter
	int h1st[ARRAY_SIZE]; // # of frames where 1st 64bit is not a header by sampling parameter
    int times[5]; // start time  yyyy, ddd, hh, mm, ss
    int timee[5]; // end time  yyyy, ddd, hh, mm, ss
	int mmdds[2]; // start date month mday
	int mmdde[2]; // end date month mday
	double period;  // period in hour (end time - start time)

	int badscan[ARRAY_SIZE]; // # of scans where error occurred during sampling by sampling parameters
	int err16[ARRAY_SIZE]; // # of error 16 occurrences during sampling by sampling parameters
	int err19[ARRAY_SIZE]; // # of error 19 occurrences during sampling by sampling parameters
	int err20[ARRAY_SIZE]; // # of error 20 occurrences during sampling by sampling parameters
	int others[ARRAY_SIZE]; // # of other errors occurrences during sampling by sampling parameters
	int errall[ARRAY_SIZE]; // # of total errors occurrences during sampling by sampling parameters

    int skfmin,skfmax;  // to keep actual sampling frequency minimum and maximum (kHz)

    DATACHK_LOG_T  dl;
    SAMPLING_LOG_T sl;
} VSSPLOGANA_INTERNAL_USE_T;

typedef struct logparam
{
   int  n;          //number of param
   char param[60][128];   // 60x128 characters
} logparam_t;

int leaf_year_check(int year)   // return 0 in case of leaf year
{
       /* leaf year check */
      if (year-(year/4)*4 == 0) 
      { 
          if (year-(year/100)*100 == 0) 
          { 
               if (year-(year/400)*400 == 0) { return 0;}
               return -1;
           }
           return 0;
       }
       return -1;
}

void ddd2md(int year, int ddd, int* month, int* mday)
{
      int md[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
      int i,td,wd ;
       /* leaf year check */
      if (leaf_year_check(year)==0)
      {
         md[1]=29;
      }
      *month=0;
      *mday=0;
      td=ddd;
      for (i=0; i< 12; i++)
      {
         wd=td-md[i];
         if (wd <=0)
         {
           *mday=td;
           *month=i+1;
           goto exithere;
          }
         td=wd;
       }
   exithere:;
 }

void conv2n1n2n3(int fsampl, int adbit, int ch,
				 int *n1, int *n2, int *n3)
{
	switch(fsampl){
	  case 2000: *n1=0; break; 
	  case 4000: *n1=1; break; 
	  case 8000: *n1=2; break; 
	  case 16000: *n1=3; break; 
	  case 32000: *n1=4; break; 
	  case 64000: *n1=5; break;
	  case 128000: *n1=6; break;
	  default: *n1=-1;
	}
	switch(adbit){
	  case 1: *n2=0; break; 
	  case 2: *n2=1; break; 
	  case 4: *n2=2; break; 
	  case 8: *n2=3; break; 
	  default: *n2=-1;
	}
	switch(ch){
	  case 1: *n3=0; break; 
	  case 4: *n3=1; break; 
	  case 2: *n3=2; break; 
	  default: *n3=-1;
	}
	;
}

int logline_parse(char *line, logparam_t *lp)
{
/*
    Parse 1 line data into each item, where ' ' is separator.
    Total number of items is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.


*/
    char abuf[500];
    char *cset=" ";
    char *p=NULL;
    int k,len;
   
    len=strlen(line);
    if(len <= 0) return 0;
    k=0;
    sprintf(abuf,"%s",strtok(line, cset));
    strcpy(lp->param[k],abuf);  
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        strcpy(lp->param[k],abuf);  
        k++;
    }
    lp->n=k;
    return k;
}


int logline_parse2(char *line, logparam_t *lp)  //2005-07-25
{
/*
    Parse 1 line data into each item, where ' ' is separator.
    Total number of items is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.


*/
    char abuf[500];
    char *cset="=";
    char *p=NULL;
    int k,len;
   
    len=strlen(line);
    if(len <= 0) return 0;
    k=0;
    sprintf(abuf,"%s",strtok(line, cset));
    strcpy(lp->param[k],abuf);  
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        strcpy(lp->param[k],abuf);  
        k++;
    }
    lp->n=k;
    return k;
}

void cline2time(char *line, int *time)
{
      //; time  intarr(5)
	int k,len;
	char cbuf[60],hhmmss[60];
    logparam_t lp;
    k=logline_parse(line, &lp);
    // check yearday include '/'   yyyy/ddd
	len=strlen(lp.param[0]);
	if((len >= 8) && (lp.param[0][4] == '/')){
		strncpy(cbuf,lp.param[0],4);   // YYYY
		cbuf[4]='\0';
		time[0]=atoi(cbuf);
		strncpy(cbuf,lp.param[0]+5,3);  // DDD
		cbuf[3]='\0';
		time[1]=atoi(cbuf);
		strcpy(hhmmss,lp.param[1]);
	} else {
		time[0]=0;
		time[1]=0;
		strcpy(hhmmss,lp.param[0]);
	}
	strncpy(cbuf,hhmmss,2); cbuf[2]='\0'; time[2]=atoi(cbuf);  // hh
	strncpy(cbuf,hhmmss+3,2); cbuf[2]='\0'; time[3]=atoi(cbuf);  // mm
	strncpy(cbuf,hhmmss+6,2); cbuf[2]='\0'; time[4]=atoi(cbuf);  // ss
}

double sajikan(int *times, int *timee)
{
 /*    ; times(5) ---  yyyy, ddd, hh, mm, ss
     ; timee(5) ---  yyyy, ddd, hh, mm, ss
     ; do not cross year!!
 */
    double hour;
	int secs,sece;
    secs=times[1]*86400+times[2]*3600+times[3]*60+times[4];
    sece=timee[1]*86400+timee[2]*3600+timee[3]*60+timee[4];
    hour=(double)(sece-secs)/3600.0;
    return hour;
}

int sa_sec(int *times, int *timee)
{
 /*    ; times(5) ---  yyyy, ddd, hh, mm, ss
     ; timee(5) ---  yyyy, ddd, hh, mm, ss
     ; do not cross year!!
 */
	int secs,sece,sec;
    secs=times[1]*86400+times[2]*3600+times[3]*60+times[4];
    sece=timee[1]*86400+timee[2]*3600+timee[3]*60+timee[4];
    sec=sece-secs;
    return sec;
}

void timefield2time(char *line, int *time)
{
	/*
      ; time  intarr(5)  yyyy ddd hh mm ss
	  */
    char c1,c2;
	char cbuf[10];
	strncpy(cbuf,line,2); cbuf[2]='\0'; time[0]=atoi(cbuf)+2000;  // yyyy
	strncpy(cbuf,line+2,3); cbuf[3]='\0'; time[1]=atoi(cbuf);  // ddd
	strncpy(cbuf,line+5,2); cbuf[2]='\0'; time[2]=atoi(cbuf);  // hh
	strncpy(cbuf,line+7,2); cbuf[2]='\0'; time[3]=atoi(cbuf);  // mm
	strncpy(cbuf,line+9,2); cbuf[2]='\0'; time[4]=atoi(cbuf);  // ss
	c1=line[11];
 	c2=line[12];
}

void sample_param_field(char *line, int *span, int *fsampl, int *adbit, int *ch)
{
	// analyze sampling parameter in "samplig" log file
    //char *cset="=";
    int k;
    logparam_t lp;
    logparam_t lp2;
    k=logline_parse(line, &lp);

	// span get
	k=logline_parse2(lp.param[3], &lp2);
	*span=atoi(lp2.param[1]);
	// fsampl get
	k=logline_parse2(lp.param[4], &lp2);
	*fsampl=atoi(lp2.param[1]);

	// adbit get
	k=logline_parse2(lp.param[5], &lp2);
	*adbit=atoi(lp2.param[1]);

	// ch get
	k=logline_parse2(lp.param[6], &lp2);
	*ch=atoi(lp2.param[1]);
}



void anallogfile1(VSSPLOGANA_INTERNAL_USE_T *vt) 
{
	/* analyze sampling log file */
    int k,n;
	int n1,n2,n3;
	int adbit,ch,fsampl;
	int span, flgerr, errorn;
	int time[5];
	double mbps;
    int nscan;

    char line[500];
    logparam_t lp;

	vt->nscantotal=0;
    nscan=0;
	while(1){       // scan loop entry
	  flgerr=0;
loop_in_scan:
      if( fgets(line, 500, vt->fin ) == NULL) break;
      if(strncmp(line+14,"sampling start",14)==0) {
		  vt->nscantotal++;
          nscan++;
          if(nscan > MAX_SCAN){
              printf("Warning!! # of scans exceed program limit %d\n",MAX_SCAN);
              nscan=MAX_SCAN;
          }

          vt->sl.nscan=nscan;
          vt->sl.scan[nscan-1].adbit=0;
          vt->sl.scan[nscan-1].sfreq=0;
          vt->sl.scan[nscan-1].nch=0;
          vt->sl.scan[nscan-1].span=0;
          vt->sl.scan[nscan-1].flag_lpf=0;
          vt->sl.scan[nscan-1].badscan=0;
          vt->sl.scan[nscan-1].err16=0;
          vt->sl.scan[nscan-1].err19=0;
          vt->sl.scan[nscan-1].err20=0;
          vt->sl.scan[nscan-1].errall=0;
          vt->sl.scan[nscan-1].others=0;

		  goto l_sampling_param;
	  }
      if(strncmp(line+14,"sampling stop",13)==0) goto l_sampling_stop;  
      if(strncmp(line+25,"read() failed! Error",20)==0) goto l_error_code;  
	  goto loop_in_scan;

l_sampling_stop:
      timefield2time(line, vt->timee);
	  continue;

l_sampling_param:
      timefield2time(line, time);
      sample_param_field(line, &span, &fsampl, &adbit, &ch);
	  //printf("Debug span,fsampl,adbit,ch %d %d %d %d\n",span,fsampl,adbit,ch);
      memcpy(vt->sl.scan[nscan-1].times, time, 5*sizeof(int));
      vt->sl.scan[nscan-1].adbit=adbit;
      if(fsampl > 0){
          vt->sl.scan[nscan-1].sfreq=fsampl;
          vt->sl.scan[nscan-1].flag_lpf=1;
      } else {
          vt->sl.scan[nscan-1].sfreq=-fsampl;
          vt->sl.scan[nscan-1].flag_lpf=0;
      }
      vt->sl.scan[nscan-1].nch=ch;
      vt->sl.scan[nscan-1].span=span;

	  fsampl*=1000;
	  if(fsampl==0) continue;
	   
      if(fsampl < vt->skfmin) vt->skfmin=fsampl;
      if(fsampl > vt->skfmax) vt->skfmax=fsampl;

	  if(vt->nscantotal==1){
		  for(n=0; n<5; n++) vt->times[n]=time[n];
		  vt->spanmin=span;
		  vt->spanmax=span;
	  }
	  if(span <  vt->spanmin) vt->spanmin=span;
	  if(span >  vt->spanmax) vt->spanmax=span;
      mbps=(double)fsampl*(double)adbit*(double)ch/1000.0;
      conv2n1n2n3(fsampl,adbit,ch,&n1,&n2,&n3);
	  vt->nscan[N123(n1,n2,n3)]++;
	  goto loop_in_scan;

l_error_code:
	  if(flgerr==0){
		  vt->badscan[N123(n1,n2,n3)]++;
		  flgerr=1;
	  }
	  // error number get
      k=logline_parse(line, &lp);
	  errorn=atoi(lp.param[7]);
	  switch(errorn){
	    case 16: 
            vt->err16[N123(n1,n2,n3)]++; 
            vt->sl.scan[nscan-1].err16=1;
            break; 
	    case 19: 
            vt->err19[N123(n1,n2,n3)]++; 
            vt->sl.scan[nscan-1].err19=1;
            break; 
	    case 20: 
            vt->err20[N123(n1,n2,n3)]++; 
            vt->sl.scan[nscan-1].err20=1;
            break; 
	    default: 
            vt->others[N123(n1,n2,n3)]++; 
            vt->sl.scan[nscan-1].others=1;
            break; 
	  }
      vt->errall[N123(n1,n2,n3)]++;
      vt->sl.scan[nscan-1].errall=1;
	  goto loop_in_scan;
	}
    ddd2md(vt->times[0], vt->times[1], &vt->mmdds[0], &vt->mmdds[1]);
    if(vt->times[0]==0) vt->mmdds[0]=0;
    ddd2md(vt->timee[0], vt->timee[1], &vt->mmdde[0], &vt->mmdde[1]);
    if(vt->timee[0]==0) vt->mmdde[0]=0;
    vt->period=sajikan(vt->times,vt->timee);
}


void anallogfile2(VSSPLOGANA_INTERNAL_USE_T *vt) 
{
	/* analyze datachk summary out log file */
    int k,n;
	int n1,n2,n3;
	//int k1,k2,k3;
	int i1,i2,i3,i4,i5,i6,i7,i8;
	int adbit,ch,fsampl,filter;
	int span, boffset, nslip, numslip;
	int time[5];
	double mbps,tslip;
    int nscan;

    char line[500];
    logparam_t lp;

	vt->nscantotal=0;
    nscan=0;
	while(1){
	  numslip=0;
      if( fgets(line, 500, vt->fin ) == NULL) break;
      if(strncmp(line,"# File",6)==0) {
		  vt->nscantotal++;
          nscan++;
          if(nscan > MAX_SCAN){
              printf("Warning!!  # of scans exceed program limit %d\n",MAX_SCAN);
              nscan=MAX_SCAN;
          }
          vt->dl.nscan=nscan;
          vt->dl.scan[nscan-1].adbit=0;
          vt->dl.scan[nscan-1].sfreq=0;
          vt->dl.scan[nscan-1].nch=0;
          vt->dl.scan[nscan-1].span=0;
          vt->dl.scan[nscan-1].frames=0;
          vt->dl.scan[nscan-1].slipfram=0;
          vt->dl.scan[nscan-1].tdiscon=0;
          vt->dl.scan[nscan-1].tdiscon2=0;
          vt->dl.scan[nscan-1].auxerr=0;
          vt->dl.scan[nscan-1].auxmis=0;
          vt->dl.scan[nscan-1].bitrev=0;
          vt->dl.scan[nscan-1].eflg=0;
          vt->dl.scan[nscan-1].h1st=0;
          vt->dl.scan[nscan-1].slip=0;
          vt->dl.scan[nscan-1].k5format=0;
          vt->dl.scan[nscan-1].lpf=0;
		  continue;
	  }
      if(strncmp(line,"# FMT",5)==0) goto l_sampling_param;  
      if(strncmp(line,"# Start",7)==0) goto l_start_and_end;  
      if(strncmp(line,"# Du",4)==0) goto l_duration;  
      if(strncmp(line,"# Byte",6)==0) goto l_1stheader;  
      if(strncmp(line,"# STATISTICS",12)==0) goto l_statistics;  
      if(strncmp(line,"# BIT",5)==0) goto l_bit_slip; 
	  continue;

l_sampling_param:
      if( fgets(line, 500, vt->fin ) == NULL) break;
      k=logline_parse(line, &lp);
	  adbit = atoi(lp.param[1]);
	  ch    = atoi(lp.param[2]);
	  fsampl= atoi(lp.param[3]);
      filter=0;
	  if(k >= 5) filter=atoi(lp.param[4]);
      mbps=(double)fsampl*(double)adbit*(double)ch/1000.0;

      vt->dl.scan[nscan-1].k5format=1;
      if(strncmp(lp.param[0],"VS32",4)==0) vt->dl.scan[nscan-1].k5format=2;
      if(strncmp(lp.param[0],"VS64",4)==0) vt->dl.scan[nscan-1].k5format=3;

      vt->dl.scan[nscan-1].adbit=adbit;
      vt->dl.scan[nscan-1].sfreq=fsampl/1000;
      vt->dl.scan[nscan-1].nch=ch;
      vt->dl.scan[nscan-1].lpf=filter;

      conv2n1n2n3(fsampl,adbit,ch,&n1,&n2,&n3);
	  if(n1 == -1 || n2 == -1 || n3 == -1){
          // skip data untill next scan data
//		  while(1){
//             if( fgets(line, 500, vt->fin ) == NULL) break;
//             if(strncmp(line,"# File",6)==0) {
//		        vt->nscantotal++;
//		        break;
//			 }
//		  }
		  continue;
      } else {
         if(fsampl < vt->skfmin) vt->skfmin=fsampl;
         if(fsampl > vt->skfmax) vt->skfmax=fsampl;

	     vt->nscan[N123(n1,n2,n3)]++;
      }
	  continue;

l_start_and_end:
      if( fgets(line, 500, vt->fin ) == NULL) break;
	  // start time
      cline2time(line, time);
	  if(vt->nscantotal==1){
		  for(n=0; n<5; n++) vt->times[n]=time[n];
	  }

      memcpy(vt->dl.scan[nscan-1].times,time,5*sizeof(int));

	  // end time
      if( fgets(line, 500, vt->fin ) == NULL) break;
      cline2time(line, time);
	  for(n=0; n<5; n++) vt->timee[n]=time[n];

      vt->dl.scan[nscan-1].span=sa_sec(vt->dl.scan[nscan-1].times,time);


	  continue;

l_duration:
      if( fgets(line, 500, vt->fin ) == NULL) break;
	  span=atoi(line);
	  if(vt->nscantotal==1){
		  //vt->spanmin=span;
		  //vt->spanmax=span;
		  vt->spanmin=9999;
		  vt->spanmax=0;
	  }
      vt->dl.scan[nscan-1].span=span;
      if(span > 0){
	      if(span <  vt->spanmin) vt->spanmin=span;
	      if(span >  vt->spanmax) vt->spanmax=span;
      }
	  continue;

l_1stheader:
      if( fgets(line, 500, vt->fin ) == NULL) break;
	  boffset=atoi(line);
	  if(boffset > 0){
		  vt->h1st[N123(n1,n2,n3)]++;
          vt->dl.scan[nscan-1].h1st=1;
	  }
	  continue;

l_statistics:
      if( fgets(line, 500, vt->fin ) == NULL) break;
      k=logline_parse(line, &lp);
	  i6=0;
	  i1=atoi(lp.param[0]);  // total # frame
	  i2=atoi(lp.param[1]);  // bit slip or make frames
	  i3=atoi(lp.param[2]);  // time discon frames
	  i4=atoi(lp.param[3]);  // time discon and bit slip or make frames
	  i5=atoi(lp.param[4]);  // header separation frames
	  if(k >=6 ) i6=atoi(lp.param[5]); // EFLG occurred frames
	  if(k >=7 ) i7=atoi(lp.param[6]); // 1 bit reversal in a sync field
	  if(k >=8 ) i8=atoi(lp.param[7]); // AUX field misalignment
	  vt->frames[N123(n1,n2,n3)]+=i1;
	  vt->slipfram[N123(n1,n2,n3)]+=i2;
	  vt->tdiscon[N123(n1,n2,n3)]+=i3;
	  vt->tdiscon2[N123(n1,n2,n3)]+=i4;
	  vt->auxerr[N123(n1,n2,n3)]+=i5;
	  vt->eflg[N123(n1,n2,n3)]+=i6;
	  vt->bitrev[N123(n1,n2,n3)]+=i7;
	  vt->auxmis[N123(n1,n2,n3)]+=i8;

      vt->dl.scan[nscan-1].frames=i1;
      vt->dl.scan[nscan-1].slipfram=i2;
      vt->dl.scan[nscan-1].tdiscon=i3;
      vt->dl.scan[nscan-1].tdiscon2=i4;
      vt->dl.scan[nscan-1].auxerr=i5;
      vt->dl.scan[nscan-1].eflg=i6;
      vt->dl.scan[nscan-1].bitrev=i7;
      vt->dl.scan[nscan-1].auxmis=i8;

	  continue;

l_bit_slip:
	  // followings are slip time statistics
	  while(1){
         if( fgets(line, 500, vt->fin ) == NULL) break;
         if(strncmp(line,"# File",6)==0) {
		     vt->nscantotal++;
			 break;
		 }
         k=logline_parse(line, &lp);
		 if(k==0) continue;
		 i1=atoi(lp.param[0]);
		 nslip=atoi(lp.param[1]);
		 if((abs(nslip)-256)==(int)(mbps*1.0e6)) continue;  // this is not bit slip
         tslip=(double)nslip/(mbps*1.0e6);  // slip length in sec
		 numslip++;
		 if(numslip == 1){
            printf(" SCAN # %d   SLIP DETECTED!\n",vt->nscantotal);
            //printf("  %g %d %g\n",mbps,nslip,tslip);  //debug
            vt->dl.scan[nscan-1].slip++;
		 }
	  }
   }
   ddd2md(vt->times[0], vt->times[1], &vt->mmdds[0], &vt->mmdds[1]);
   if(vt->times[0]==0) vt->mmdds[0]=0;
   ddd2md(vt->timee[0], vt->timee[1], &vt->mmdde[0], &vt->mmdde[1]);
   if(vt->timee[0]==0) vt->mmdde[0]=0;
   vt->period=sajikan(vt->times,vt->timee);

}



void anallogfile(VSSPLOGANA_INTERNAL_USE_T *vt) 
{
    /* analyze samplig log file */
	
    int i;
    char line[500];

    /* log file open  */

    if( (vt->fin  = fopen( vt->fname, "r" )) == NULL ) {
        printf("File %s Open Error! \n",vt->fname);
        goto exitend2;
	}
    printf("FILE : %s \n",vt->fname);

	/* check 1st line of log file */
	   /*   if 1st line starting with '# File Name'
	        it is log file of 'datachk'            */
    if( fgets(line, 500, vt->fin ) == NULL) goto exitend;
	vt->logmode=1;
    if(strncmp(line,"# File",6)==0) vt->logmode=2;  // This is "datachk" out log
    rewind(vt->fin);

	if(vt->logmode==2){  // datachk log
       anallogfile2(vt);
	} else {             // sampling log
       anallogfile1(vt);
	}
 exitend:
    i=fclose( vt->fin ) ;
exitend2:;
}

void initialize_viu(VSSPLOGANA_INTERNAL_USE_T *vt)
{
	int n;
    // initialize internal variables 
	vt->k5format=0;
	vt->logmode=0;
	vt->spanmax=0;
	vt->spanmin=0;
	vt->nscantotal=0;
    vt->skfmin=128000;
    vt->skfmax=0;

	for(n=0; n<ARRAY_SIZE; n++){
		vt->nscan[n]=0;
		vt->frames[n]=0;
		vt->slipfram[n]=0;
		vt->tdiscon[n]=0;
		vt->tdiscon2[n]=0;
		vt->auxerr[n]=0;
		vt->auxmis[n]=0;
		vt->bitrev[n]=0;
		vt->eflg[n]=0;
		vt->h1st[n]=0;
		vt->badscan[n]=0;
		vt->err16[n]=0;
		vt->err19[n]=0;
		vt->err20[n]=0;
		vt->others[n]=0;
		vt->errall[n]=0;
	}
	for(n=0; n<5; n++){
		vt->times[n]=0;
		vt->timee[n]=0;
	}
	for(n=0; n<2; n++){
		vt->mmdds[n]=0;
		vt->mmdde[n]=0;
	}
	vt->period=0.0;
}

void displogmode1(VSSPLOGANA_INTERNAL_USE_T *vt)
{
	/* display sampling log analysis result */
	  int n1,n2,n3;
      int sftable[]={2,4,8,16,32,64,128};
      int adtable[]={1,2,4,8};
      int chtable[]={1,4,2};
      int n1max,n1min;
      int n3max,n3min;

      n1min=0;
      n1max=6;
      switch(vt->skfmin){
        case 2000: n1min=0; break; 
	    case 4000: n1min=1; break; 
	    case 8000: n1min=2; break; 
	    case 16000: n1min=3; break; 
	    case 32000: n1min=4; break; 
	    case 64000: n1min=5; break;
	    case 128000: n1min=6; break;
	    default: n1min=0;
	  }
      switch(vt->skfmax){
        case 2000: n1max=1; break; 
	    case 4000: n1max=2; break; 
	    case 8000: n1max=3; break; 
	    case 16000: n1max=4; break; 
	    case 32000: n1max=5; break; 
	    case 64000: n1max=6; break;
	    case 128000: n1max=7; break;
	    default: n1max=6;
	  }

      n3min=0; n3max=2;   // change this when 2ch mode is used

      printf("\n");
      printf("******** SUMMARY OF K5/VSSP32 SAMPLING LOG FILE ANALYSIS **************\n");
      printf("  FILE NAME         : %s\n", vt->fname);
      printf("  SCAN LENGTH (sec) : MIN = %5d       MAX = %5d\n",vt->spanmin,vt->spanmax);
      printf("  TOTAL # of SCANS  : %d\n",vt->nscantotal);
      printf("  START TIME        : %04d/%02d/%02d (%03d) %02d:%02d:%02d\n",
              vt->times[0],vt->mmdds[0],vt->mmdds[1],vt->times[1],vt->times[2],vt->times[3],vt->times[4]);
	  printf("  END   TIME        : %04d/%02d/%02d (%03d) %02d:%02d:%02d\n",
              vt->timee[0],vt->mmdde[0],vt->mmdde[1],vt->timee[1],vt->timee[2],vt->timee[3],vt->timee[4]);
      printf("  PERIOD (hour)     : %f\n",vt->period);
      printf("=======================================================================\n");
      printf("   SAMPLER MODE   # of SCANs                # of ERRORS\n");
      printf(" fs(MHz) #AD #CH   ALL   Bad       ALL   Err16   Err19   Err20  Others\n");
      printf("-----------------------------------------------------------------------\n");
      n2=0;
      //for(n1=0; n1<6; n1++){
      for(n1=n1min; n1<n1max; n1++){
		  for(n3=n3min; n3<n3max; n3++){
              if(vt->nscan[N123(n1,n2,n3)] > 0){
                 printf("%5d%6d%4d%6d%6d   %8d%8d%8d%8d%8d\n",
			        sftable[n1],adtable[n2],chtable[n3],
                    vt->nscan[N123(n1,n2,n3)], vt->badscan[N123(n1,n2,n3)],
				    vt->errall[N123(n1,n2,n3)],vt->err16[N123(n1,n2,n3)],
                    vt->err19[N123(n1,n2,n3)],vt->err20[N123(n1,n2,n3)],
                    vt->others[N123(n1,n2,n3)]);
              }
		  }
	  }
      printf("=======================================================================\n");
      printf(" Note: Err16  -- DMA process error\n");
      printf("       Err19  -- Header is not found at expected position in data\n");
      printf("       Err20  -- Header-like data is found in first garbage data\n");
      printf("       Others -- Other error code\n");
      printf("\n");
}


void displogmode2(VSSPLOGANA_INTERNAL_USE_T *vt)
{
	/* display datachk summary log analysis result */
	  int n1,n2,n3;
      int sftable[]={2,4,8,16,32,64,128};
      int adtable[]={1,2,4,8};
      int chtable[]={1,4,2};
      int n1max,n1min;
      int n3max,n3min;

      n1min=0;
      n1max=6;
      switch(vt->skfmin){
        case 2000: n1min=0; break; 
	    case 4000: n1min=1; break; 
	    case 8000: n1min=2; break; 
	    case 16000: n1min=3; break; 
	    case 32000: n1min=4; break; 
	    case 64000: n1min=5; break;
	    case 128000: n1min=6; break;
	    default: n1min=0;
	  }
      switch(vt->skfmax){
        case 2000: n1max=1; break; 
	    case 4000: n1max=2; break; 
	    case 8000: n1max=3; break; 
	    case 16000: n1max=4; break; 
	    case 32000: n1max=5; break; 
	    case 64000: n1max=6; break;
	    case 128000: n1max=7; break;
	    default: n1max=6;
	  }

      n3min=0; n3max=2;   // change this when 2ch mode is used

     
      printf("\n");
      printf("*************** SUMMARY OF DATACHK LOG FILE ANALYSIS ********************\n");
      printf("  FILE NAME         : %s\n", vt->fname);
      printf("  SCAN LENGTH (sec) : MIN = %5d       MAX = %5d\n",vt->spanmin,vt->spanmax);
      printf("  TOTAL # of SCANS  : %d\n",vt->nscantotal);
      printf("  START TIME        : %04d/%02d/%02d (%03d) %02d:%02d:%02d\n",
              vt->times[0],vt->mmdds[0],vt->mmdds[1],vt->times[1],vt->times[2],vt->times[3],vt->times[4]);
	  printf("  END   TIME        : %04d/%02d/%02d (%03d) %02d:%02d:%02d\n",
              vt->timee[0],vt->mmdde[0],vt->mmdde[1],vt->timee[1],vt->timee[2],vt->timee[3],vt->timee[4]);
      printf("  PERIOD (hour)     : %f\n",vt->period);
      printf("===============================================================================\n");
      printf("   SAMPLER MODE   # of SCANs   TOTAL              # of BAD FRAMES\n");
      //printf(" fs(MHz) #AD #CH   ALL  Herr   FRAME   SLIP  T_DIS T_W_BS AUXERR   EFLG  B_REV\n");
      printf(" fs(MHz) #AD #CH   ALL  Herr   FRAME   SLIP  T_DIS T_W_BS AUXMIS   EFLG  B_REV\n");
      printf("-------------------------------------------------------------------------------\n");
      n2=0;
      for(n1=n1min; n1<n1max; n1++){
		  for(n3=n3min; n3<n3max; n3++){
              if(vt->nscan[N123(n1,n2,n3)] > 0){
                   printf("%5d%6d%4d%6d%6d %7d%7d%7d%7d%7d%7d%7d\n",
			           sftable[n1],adtable[n2],chtable[n3],
                         vt->nscan[N123(n1,n2,n3)], vt->h1st[N123(n1,n2,n3)],
				         vt->frames[N123(n1,n2,n3)],vt->slipfram[N123(n1,n2,n3)],
                         vt->tdiscon[N123(n1,n2,n3)],vt->tdiscon2[N123(n1,n2,n3)],
                         //vt->auxerr[N123(n1,n2,n3)],vt->eflg[N123(n1,n2,n3)],
                         vt->auxmis[N123(n1,n2,n3)],vt->eflg[N123(n1,n2,n3)],
				         vt->bitrev[N123(n1,n2,n3)]);
              }
		  }
	  }
      printf("===============================================================================\n");
      printf(" Note: Herr   -- # of scans where 1st 64bit is not a header\n");
      printf("       SLIP   -- # of frames where bit lack or make occurred\n");
      printf("       T_DIS  -- # of frames where time discontinuity occurred\n");
      printf("       T_W_BS -- # of frames where both time discontinuity and\n");
      printf("                   bit slip or make occurred\n");
      //printf("       AUXERR -- # of frames where AUX field in header has error\n");
      printf("       AUXMIS -- # of frames where AUX field misalignment occurred (data is OK)\n");
      printf("       EFLG   -- # of frames where EFLG (burst error) detected\n");
      printf("       B_REV  -- # of frames where one bit reversal in a sync field detected\n");
      printf("\n");
}


void compare_two(VSSPLOGANA_INTERNAL_USE_T *vt)
{
/*

    campare sampling log and datachk log
   
*/

    int i,n;
    int nscan_sl,nscan_dl;
    int sec_haba=3;
    int sasec;
    int lostscan,mismatch;
    int iok,iok2;

    DATACHK_LOG_SCAN_T  *ds;
    SAMPLING_LOG_SCAN_T *ss;
    ds=vt->dl.scan;
    ss=vt->sl.scan;

    nscan_sl=vt->sl.nscan;
    nscan_dl=vt->dl.nscan;

    lostscan=0;
    mismatch=0;

//    for(n=0; n<nscan_sl; n++){
//        iok=0;  iok2=0;
//        for(i=0; i<nscan_dl; i++){
//            sasec=sa_sec(ss[n].times,ds[i].times);
//            if(abs(sasec) < sec_haba){   // find sampled data
//                iok++;
//                if(ss[n].adbit==ds[i].adbit && ss[n].sfreq==ds[i].sfreq && ss[n].nch== ds[i].nch){
//                    iok2=1;
//                }
//                break;
//
//            }
//        }
//        if(iok==0) lostscan++;
//        //if(iok2==0) mismatch++;
//   }

    lostscan=nscan_sl-nscan_dl;

//    printf("Total # of scans : %d\n",nscan_sl);
//    if(lostscan > 0){
//        printf("  %d scans out of %d scans are missing!\n",lostscan,nscan_sl);
//    }



    for(i=0; i<nscan_dl; i++){
        iok=0;  iok2=0;
        for(n=0; n<nscan_sl; n++){
            sasec=sa_sec(ss[n].times,ds[i].times);
            if(abs(sasec) < sec_haba){   // find sampled data
                if(ss[n].adbit==ds[i].adbit && ss[n].sfreq==ds[i].sfreq && ss[n].nch== ds[i].nch){
                    iok2=1;
                }
                break;

            }
        }
        if(iok2==0) mismatch++;
    }

    printf("Total # of scans \"sampling\" executed : %d\n",nscan_sl);
    printf("Total # of scans \"datachk\" analyzed  : %d\n",nscan_dl);
    printf("   %d scans out of %d scans are lost!\n",lostscan,nscan_sl);
    printf("   %d scans out of %d scans have wrong sampling information!\n",mismatch,nscan_dl);
}






int main(int argc, char *argv[])
{
    char* dflt_fname="tds.data";
    int n;
    int iok1,iok2;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    VSSPLOGANA_INTERNAL_USE_T vt;
    //DATACHK_LOG_T  dl;
    //SAMPLING_LOG_T sl;

    logparam_t logname;
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
       //strcpy(fname,dflt_fname);
       printf("%s %s\n",PROGRAM_NAME,PROGRAM_VERSION);
       printf("How to run: \n");
       printf("  vssplogana logfile1 [logfile2 [....]]\n");
       printf("\n");
       printf("    where  logfile1 ----- log file name of sampling out\n");
       printf("                           or datachk out\n");
       printf("           logfile2 ----- log file name of sampling out\n");
       printf("                           or datachk out\n");
       exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("%s %s\n",PROGRAM_NAME,PROGRAM_VERSION);
       exit(0);
    }


    vt.dl.scan=(DATACHK_LOG_SCAN_T *)malloc(MAX_SCAN*sizeof(DATACHK_LOG_SCAN_T));
    vt.sl.scan=(SAMPLING_LOG_SCAN_T *)malloc(MAX_SCAN*sizeof(SAMPLING_LOG_SCAN_T));


	n=0;
    while(argc >= n+2){
       strcpy(logname.param[n],*(argv+n+1));
	   n++;
	   logname.n=n;
	}

	//for(n=0; n<logname.n; n++){
	//	printf("%d  %s\n",n,logname.param[n]);
	//}

    cstart1sec = clock();

    iok1=0;  iok2=0;
    printf("************ %s  %s ************\n",PROGRAM_NAME,PROGRAM_VERSION);
	for(n=0; n<logname.n; n++){
		strcpy(vt.fname,logname.param[n]);
        // initialize internal variables 
        initialize_viu(&vt);

        anallogfile(&vt);
	    if(vt.logmode==2){  // datachk log
           displogmode2(&vt);
           iok1=1;
		} else {
           displogmode1(&vt);  // sampling log
           iok2=1;
		}
	}

    if(iok1==1 && iok2==1){   // comparison
        compare_two(&vt);
    }

    free(vt.dl.scan);
    free(vt.sl.scan);

	cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    //printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
